import React, { useContext, useEffect } from 'react'
import PropTypes, { node } from 'prop-types'
import { GraphContext, getRelativePosition } from '../utils/index.js'
import { updateSelectedState, savePreviousSelection } from '../utils/selectable.js'
import { multiSelectCursorEffect, selectCursorEffect } from '../utils/selectable.js'
import * as d3 from 'd3-selection'


const Selectable = ({
    children = null
}) => {
    const {canvas, simulation, nodes} = useContext(GraphContext)

    useEffect(() => multiSelectCursorEffect(canvas))
    useEffect(() => selectCursorEffect(canvas, simulation))
    useEffect(() => {
        const container = d3.select(canvas.current)
        const ctx = canvas.current.getContext('2d')
        let brushing = false
        let selection = []
        let start = []

        container.on('click', () => {
            const {transform} = simulation
            const {x, y} = getRelativePosition(d3.event.x, d3.event.y, canvas.current)
            const transformed = transform.invert([x, y])
            const node = simulation.find(...transformed, 20)

            if (!d3.event.shiftKey) nodes.forEach(node => node.selected = false)
            if (node) node.selected = !node.selected
        })

        container.on('mousedown', () => {
            if (d3.event.shiftKey && container.style('cursor') !== 'pointer') {
                const {x, y} = getRelativePosition(d3.event.x, d3.event.y, canvas.current)

                start = [x, y]
                selection[0] = [x, y]
                selection[1] = [x, y]
                brushing = true
                simulation.alphaTarget(0.1).restart()
                nodes.forEach(savePreviousSelection)
            }
        })

        container.on('mousemove', () => {
            if (brushing) {
                const {x, y} = getRelativePosition(d3.event.x, d3.event.y, canvas.current)

                selection = [
                    [Math.min(start[0], x), Math.min(start[1], y)],
                    [Math.max(start[0], x), Math.max(start[1], y)]
                ]

                nodes.forEach(node => {
                    const { transform } = simulation

                    updateSelectedState(node, [
                        transform.invert(selection[0]),
                        transform.invert(selection[1])
                    ])
                })
            }
        })

        container.on('mouseup', () => {
            brushing = false
            simulation.alphaTarget(0)
        })

        simulation.on('tick.selection', () => {
            if (brushing && selection.length === 2) {
                ctx.save()
                ctx.fillStyle = 'rgba(0, 0, 0, 0.25)'
                ctx.fillRect(
                    selection[0][0],
                    selection[0][1],
                    selection[1][0] - selection[0][0],
                    selection[1][1] - selection[0][1],
                )
                ctx.restore()
            }
        })
    })

    return children
}

Selectable.propTypes = {
    children: PropTypes.node
}

export default Selectable