import * as d3 from 'd3-selection'
import { getRelativePosition } from './index.js'

/**
 * Update a node's selected state based on the current brush area.
 * 
 * @param  {object}  d      - Data backing the node
 * @param  {array}   bounds - Extent of brushed area
 * @return {boolean}        - New selected state of node
 */
export const updateSelectedState = (d, bounds) => {
    const x = bounds[0][0] <= d.x && d.x < bounds[1][0]
    const y = bounds[0][1] <= d.y && d.y < bounds[1][1]
    const selected = x && y

    if (selected && d.previouslySelected) return d.selected = false
    return d.selected = selected || d.previouslySelected
}

/**
 * Save the previous selected state to differentiate between it and temporary
 * selection changes.
 * 
 * @param {object} d - The data backing a node
 */
export const savePreviousSelection = d => {
    d.previouslySelected = d.selected
}

/**
 * Change the cursor when a user hits shift over the given `ref`.
 *
 * @param  {object}   ref - A valid react ref to an element
 * @return {function}	  - A cleanup function to remove event listeners
 */
export const multiSelectCursorEffect = (ref) => {
    const container = d3.select(ref.current)
    const cursor = container.style('cursor')

    const changeCursor = e => {
        if (e.key === 'Shift' && container.style('cursor') !== 'pointer') {
            container.style('cursor', 'crosshair')
        }
    }

    const revertCursor = e => {
        if (e.key === 'Shift' && container.style('cursor') !== 'pointer') {
            container.style('cursor', cursor)
        }
    }

    window.addEventListener('keydown', changeCursor)
    window.addEventListener('keyup', revertCursor)

    return () => {
        window.removeEventListener('keydown', changeCursor)
        window.removeEventListener('keyup', revertCursor)
    }
}

/**
 * Change the cursor when the user hovers over a node.
 *
 * @param  {object}   ref        - A valid react ref to an element
 * @param  {object}   simulation - A reference to the d3 force simulation
 * @return {function} 			 - A cleanup function to remove event listeners
 */
export const selectCursorEffect = (ref, simulation) => {
    const container = d3.select(ref.current)
    let cursor = container.style('cursor')

    container.on('mousemove.cursor', () => {
        const {transform} = simulation
        const {x, y} = getRelativePosition(d3.event.x, d3.event.y, ref.current)
        const transformed = transform.invert([x, y])
        const node = simulation.find(...transformed, 20)

        if (node) container.style('cursor', 'pointer')
        else if (d3.event.shiftKey) container.style('cursor', 'crosshair')
        else container.style('cursor', cursor)
    })

    return () => container.on('mousemove.cursor', null)
}